// Copyright © 2021 Finn Herzfeld <finn@janky.solutions>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

package verify

import (
	"log"

	"github.com/spf13/cobra"

	"gitlab.com/signald/signald-go/cmd/signaldctl/common"
	"gitlab.com/signald/signald-go/signald"
	"gitlab.com/signald/signald-go/signald/client-protocol/v0"
)

var (
	VerifyAccountCmd = &cobra.Command{
		Use:   "verify [phone number] [code]",
		Short: "verify an account and complete the registration process",
		PreRun: func(cmd *cobra.Command, args []string) {
			if len(args) != 2 {
				common.Must(cmd.Help())
				log.Fatal("must specify phone number and code")
			}
		},
		Run: func(_ *cobra.Command, args []string) {
			requestID := signald.GenerateID()
			err := common.Signald.RawRequest(v0.LegacyRequest{
				Type:     "verify",
				ID:       requestID,
				Username: args[0],
				Code:     args[1],
			})
			if err != nil {
				log.Fatal("error sending request to signald: ", err)
			}
			c := make(chan v0.LegacyResponse)
			go common.Signald.Listen(c)
			response := signald.GetLegacyResponse(c, requestID)
			if response.Type == "verification_succeeded" {
				log.Println("verification succeeded")
			} else {
				log.Fatalf("unexpected response from signald when requesting verification code: %+v", response)
			}
		},
	}
)
